<?php
/**
 * montonio/module-hire-purchase
 *
 * @category   montonio
 * @package    module-hire-purchase
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */
namespace Montonio\Hirepurchase\Controller\Payments;

/**
 * @inheritdoc
 */
class Index extends \Magento\Framework\App\Action\Action
{

    /**
     * @var \Magento\Sales\Model\OrderRepository
     */
    protected $orderRepository;

    /**
     * @var \Magento\Framework\Api\SearchCriteriaBuilder
     */
    protected $searchCriteriaBuilder;

    /**
     *
     * @param \Magento\Framework\App\Action\Context $context
     * @param \Montonio\Hirepurchase\Gateway\Payments\Config\Config $config
     * @param \Magento\Framework\App\Request\Http $request
     * @param \Magento\Sales\Model\OrderRepository $orderRepository
     * @param \Magento\Framework\Api\SearchCriteriaBuilder $searchCriteriaBuilder
     * @param \Magento\Framework\Controller\Result\RedirectFactory $resultRedirectFactory
     * @param \Magento\Framework\UrlInterface $urlInterface
     * @param \Magento\Checkout\Model\Session $checkoutSession
     * @param \Magento\Quote\Api\CartRepositoryInterface $quoteRepository
     * @param \Magento\Sales\Model\Order\Email\Sender\OrderSender $orderSender
     * @param \Magento\Sales\Model\Service\InvoiceService $invoiceService
     * @param \Magento\Sales\Model\Order\Email\Sender\InvoiceSender $invoiceSender
     * @param \Psr\Log\LoggerInterface $logger
     */
    public function __construct(
        \Magento\Framework\App\Action\Context $context,
        \Montonio\Hirepurchase\Gateway\Payments\Config\Config $config,
        \Magento\Framework\App\Request\Http $request,
        \Magento\Sales\Model\OrderRepository $orderRepository,
        \Magento\Framework\Api\SearchCriteriaBuilder $searchCriteriaBuilder,
        \Magento\Framework\Controller\Result\RedirectFactory $resultRedirectFactory,
        \Magento\Framework\UrlInterface $urlInterface,
        \Magento\Checkout\Model\Session $checkoutSession,
        \Magento\Quote\Api\CartRepositoryInterface $quoteRepository,
        \Magento\Sales\Model\Order\Email\Sender\OrderSender $orderSender,
        \Magento\Sales\Model\Service\InvoiceService $invoiceService,
        \Magento\Sales\Model\Order\Email\Sender\InvoiceSender $invoiceSender,
        \Psr\Log\LoggerInterface $logger
    ) {
        $this->request = $request;
        $this->config = $config;
        $this->orderRepository = $orderRepository;
        $this->searchCriteriaBuilder = $searchCriteriaBuilder;
        $this->resultRedirectFactory = $resultRedirectFactory;
        $this->urlInterface = $urlInterface;
        $this->checkoutSession = $checkoutSession;
        $this->quoteRepository = $quoteRepository;
        $this->orderSender = $orderSender;
        $this->logger = $logger;
        $this->invoiceService = $invoiceService;
        $this->invoiceSender = $invoiceSender;
        parent::__construct($context);
    }

    /**
     * Checkout page
     *
     * @return \Magento\Framework\Controller\ResultInterface
     */
    public function execute()
    {

        $data = $this->request->getParam('payment_token') ?? false;
        $process = false;
        $cancel = false;
        $url = $this->urlInterface->getUrl('checkout/cart');

        $enabled = $this->config->isActive();

        if (!$enabled || !$data) {
            return $this->resultRedirectFactory
                ->create()
                ->setUrl($url);
        }

        if ($data) {
            $environment = $this->config->getValue('environment');
            $privKey = $this->config->getValue($environment.'_private_key');
            $pubKey = $this->config->getValue($environment.'_public_key');
            try {
                $data = $this->config->decodeSignature($data, $privKey);
                if (is_object($data)) {
                    $data = (array)$data;
                    if (!$this->config->validateSignedData($data, $pubKey)) {
                        $data = false;
                    }
                }
            } catch (\Exception $e) {
                $this->logger->critical($e->getMessage());
            }
        }

        if (is_array($data)) {
            $orderId = $data['merchant_reference'] ?? false;
            //now we get the order by reference and get the payment method instance from that
            $lastRealOrder = $this->getOrderByIncrementId($orderId);

            $isOrderAvailable = (
                    !is_object($lastRealOrder) ||
                    $lastRealOrder->getData('increment_id') !== $orderId
                ) ? false : true;

            if ($isOrderAvailable && $data['status'] === 'finalized') {
                $process = true;
            }

            if ($isOrderAvailable && $data['status'] === 'abandoned') {
                $cancel = true;
            }
        }

        if ($process) {

            $methodInstance = $lastRealOrder
                ->getPayment()
                ->getMethodInstance() ?? false;

            $methodTarget = \Montonio\Hirepurchase\Model\Payment\PaymentsConfigProvider::CODE;
            $methodCode = $methodInstance->getCode() ?? false;
            $methodsAvailable = $this->config->getAvailableMethods();
            $subMethodCode = str_replace($methodTarget.'_', '', $methodCode);

            if ($methodCode && isset($methodsAvailable[$subMethodCode])) {

                $orderStatus = \Magento\Sales\Model\Order::STATE_PROCESSING;
                $orderState = \Magento\Sales\Model\Order::STATE_NEW;

                if ($orderStatus &&
                    $lastRealOrder->getState() == $orderState &&
                    $lastRealOrder->getStatus() == 'pending'
                ) {

                    $lastRealOrder
                        ->setState($orderStatus)
                        ->setStatus($orderStatus)
                        ->addStatusHistoryComment(
                            __(
                                'Status update by customer - method: %1, amount: %2',
                                $methodCode,
                                $data['amount']
                            )
                        );

                    $controlEmail = $this->config->getValue('order_email_after');
                    if ($controlEmail && $lastRealOrder->getEmailSent() === null) {
                        try {
                            $this->orderSender->send($lastRealOrder);
                            $lastRealOrder
                                ->addStatusHistoryComment(
                                    __('Order confirmation e-mail sent to customer')
                                );
                        } catch (\Exception $e) {
                            $this->logger->critical($e);
                        }
                    }

                    //invoice
                    $controlInvoice = $this->config->getValue('create_invoice') ?? false;
                    $invoiceCreated = false;
                    try {
                        if ($controlInvoice && $lastRealOrder->canInvoice()) {
                            $invoice = $this->invoiceService->prepareInvoice($lastRealOrder);
                            if ($invoice->getTotalQty()) {
                                $invoice->register();
                                $invoice->addComment(__('Invoice generated together with order confirmation from payment gateway'), false, true);
                                $invoice->save();
                                $invoiceCreated = true;
                            }
                        }
                    } catch (\Exception $e) {
                        $this->logger->critical($e);
                    }

                    //invoice e-mail
                    $controlInvoiceEmail = $this->config->getValue('invoice_email_after') ?? false;
                    try {
                        if (is_object($invoice) && $invoiceCreated && $controlInvoiceEmail) {
                            $this->invoiceSender->send($invoice);
                            $invoice->addComment(__('Invoice e-mail sent to customer'), false, true);
                            $invoice->save();
                        }
                    } catch (\Exception $e) {
                        $this->logger->critical($e);
                    }

                    $this->orderRepository->save($lastRealOrder);
                    $url = $this->urlInterface->getUrl('checkout/onepage/success');
                }

                if ($orderStatus &&
                    $lastRealOrder->getStatus() == 'processing'
                ) {
                    $url = $this->urlInterface->getUrl('checkout/onepage/success');
                }
            }
        }

        if ($cancel) {

            $quoteId = $lastRealOrder->getQuoteId();

            $orderStatus = \Magento\Sales\Model\Order::STATE_CANCELED;

            if ($lastRealOrder->getStatus() !== $orderStatus) {

                $lastRealOrder
                    ->setState($orderStatus)
                    ->setStatus($orderStatus)
                    ->cancel();

                $lastRealOrder
                    ->addStatusHistoryComment(__('Payment cancelled on payment gateway.'));

                $this->orderRepository->save($lastRealOrder);
            }

            $this->checkoutSession
                ->unsLastQuoteId()
                ->unsLastSuccessQuoteId()
                ->unsLastOrderId()
                ->unsLastRealOrderId();

            $this->messageManager->addSuccessMessage(
                __('Payment cancelled on payment gateway.')
            );

            if ($quoteId) {
                $quote = $this->quoteRepository->get($quoteId);
                $quote->setIsActive(1)->setReservedOrderId(null);
                $this->quoteRepository->save($quote);
                $this->checkoutSession->replaceQuote($quote);
            }

        }

        $result = $this->resultRedirectFactory
            ->create()
            ->setUrl($url);

        return $result;
    }

    /**
     * @param string $incrementId
     * @return \Magento\Sales\Api\Data\OrderInterface $order
     */
    private function getOrderByIncrementId($incrementId)
    {
        $this->searchCriteriaBuilder->addFilter('increment_id', $incrementId);

        $order = $this->orderRepository->getList(
            $this->searchCriteriaBuilder->create()
        );

        return $order->getFirstItem();
    }
}
