<?php
/**
 * montonio/module-hire-purchase
 *
 * @category   montonio
 * @package    module-hire-purchase
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */
declare(strict_types = 1);
namespace Montonio\Hirepurchase\Plugin\Virtual;

/**
 * Handle plugins on getting virtual instances on payment methods list
 */
class PaymentMethodList
{

    /**
     *
     * @param \Montonio\Hirepurchase\Gateway\Payments\Config\Config $config
     * @param \Magento\Payment\Helper\Data $helper
     */
    public function __construct(
        \Montonio\Hirepurchase\Gateway\Payments\Config\Config $config,
        \Magento\Payment\Helper\Data $helper
    ) {
        $this->config = $config;
        $this->paymentHelper = $helper;
    }

    /**
     *
     * @param \Magento\Payment\Model\PaymentMethodList $subject
     * @param array $result
     * @param int $storeId
     * @return array
     */
    public function afterGetList(\Magento\Payment\Model\PaymentMethodList $subject, $result)
    {
        $enabled = $this->config->isActive();
        $isGrouped = $this->config->getValue('grouped');
        if ($enabled && ! $isGrouped && ! empty($result)) {
            $tmp = [];
            $methods = $this->paymentHelper->getPaymentMethods();

            //for methods that don't have sort give them order at the end
            $i = 100000;
            foreach ($result as $k => $v) {
                $sort = $methods[$v->getCode()]['sort_order'] ?? $i;
                $tmp[$sort] = $v;
                $i ++;
            }
            ksort($tmp);
            $result = $tmp;
        }
        return $result;
    }
}
